@@ -186,7 +186,11 @@ NAVIGATION_CONFIG = [
               },
               {
                 path: '../../documentation/modules/auxiliary/admin/ldap/rbcd.md',
-                title: 'RBCD - Resource-based constrained delegation'
+                title: 'Resource-based constrained delegation (RBCD)'
+              },
+              {
+                path: 'kerberos/unconstrained_delegation.md',
+                title: 'Unconstrained delegation'
               }
             ]
           },
@@ -272,6 +276,10 @@ NAVIGATION_CONFIG = [
           {
             path: 'How-to-use-msfvenom.md',
             nav_order: 7
+          },
+          {
+            path: 'Managing-Sessions.md',
+            nav_order: 8
           }
         ]
       },
@@ -317,6 +325,9 @@ NAVIGATION_CONFIG = [
           {
             path: 'Metasploit-Web-Service.md'
           },
+          {
+            path: 'How-to-Configure-DNS.md'
+          },
           {
             title: 'Meterpreter',
             folder: 'meterpreter',
@@ -547,6 +558,9 @@ NAVIGATION_CONFIG = [
               {
                 path: 'How-to-check-Microsoft-patch-levels-for-your-exploit.md'
               },
+              {
+                path: "How-to-write-a-cmd-injection-module.md"
+              }
             ]
           },
           {
