@@ -30,7 +30,7 @@ class CommandShell
   include Rex::Ui::Text::Resource
 
   @@irb_opts = Rex::Parser::Arguments.new(
-    '-h' => [false, 'Help menu.'             ],
+    ['-h', '--help'] => [false, 'Help menu.'             ],
     '-e' => [true,  'Expression to evaluate.']
   )
 
@@ -203,6 +203,7 @@ Shell Banner:
     end
 
     print(tbl.to_s)
+    print("For more info on a specific command, use %grn<command> -h%clr or %grnhelp <command>%clr.\n\n")
   end
 
   def cmd_background_help
@@ -221,7 +222,6 @@ Shell Banner:
     end
 
     if prompt_yesno("Background session #{name}?")
-      Rex::Ui::Text::Shell::HistoryManager.pop_context
       self.interacting = false
     end
   end
@@ -236,36 +236,35 @@ Shell Banner:
   end
 
   def cmd_sessions(*args)
-    if args.length.zero? || args[0].to_i <= 0
-      # No args
-      return cmd_sessions_help
-    end
-
-    if args.length == 1 && (args[1] == '-h' || args[1] == 'help')
-      # One arg, and args[1] => '-h' '-H' 'help'
-      return cmd_sessions_help
-    end
-
     if args.length != 1
-      # More than one argument
+      print_status "Wrong number of arguments expected: 1, received: #{args.length}"
       return cmd_sessions_help
     end
 
-    if args[0].to_s == self.name.to_s
+    if args[0] == '-h' || args[0] == '--help'
+      return cmd_sessions_help
+    end
+
+    session_id = args[0].to_i
+    if session_id <= 0
+      print_status 'Invalid session id'
+      return cmd_sessions_help
+    end
+
+    if session_id == self.sid
       # Src == Dst
       print_status("Session #{self.name} is already interactive.")
     else
       print_status("Backgrounding session #{self.name}...")
-      Rex::Ui::Text::Shell::HistoryManager.pop_context
       # store the next session id so that it can be referenced as soon
       # as this session is no longer interacting
-      self.next_session = args[0]
+      self.next_session = session_id
       self.interacting = false
     end
   end
 
   def cmd_resource(*args)
-    if args.empty?
+    if args.empty? || args[0] == '-h' || args[0] == '--help'
       cmd_resource_help
       return false
     end
@@ -322,9 +321,9 @@ Shell Banner:
   end
 
   def cmd_shell(*args)
-    if args.length == 1 && (args[1] == '-h' || args[1] == 'help')
-      # One arg, and args[1] => '-h' '-H' 'help'
-      return cmd_sessions_help
+    if args.length == 1 && (args[0] == '-h' || args[0] == '--help')
+      # One arg, and args[0] => '-h' '--help'
+      return cmd_shell_help
     end
 
     if platform == 'windows'
@@ -348,7 +347,7 @@ Shell Banner:
       print_status("Using `script` to pop up an interactive shell")
       # Payload: script /dev/null
       # Using /dev/null to make sure there is no log file on the target machine
-      # Prevent being detected by the admin or antivirus softwares
+      # Prevent being detected by the admin or antivirus software
       shell_command("#{script_path} /dev/null")
       return
     end
@@ -412,7 +411,7 @@ Shell Banner:
 
   def cmd_download(*args)
     if args.length != 2
-      # no argumnets, just print help message
+      # no arguments, just print help message
       return cmd_download_help
     end
 
@@ -447,7 +446,7 @@ Shell Banner:
 
   def cmd_upload(*args)
     if args.length != 2
-      # no argumnets, just print help message
+      # no arguments, just print help message
       return cmd_upload_help
     end
 
@@ -466,9 +465,9 @@ Shell Banner:
       content = File.binread(src)
       result = _file_transfer.write_file(dst, content)
       print_good("File <#{dst}> upload finished") if result
-      print_error("Error occured while uploading <#{src}> to <#{dst}>") unless result
+      print_error("Error occurred while uploading <#{src}> to <#{dst}>") unless result
     rescue => e
-      print_error("Error occured while uploading <#{src}> to <#{dst}> - #{e.message}")
+      print_error("Error occurred while uploading <#{src}> to <#{dst}> - #{e.message}")
       elog(e)
       return
     end
@@ -548,7 +547,7 @@ Shell Banner:
     if expressions.empty?
       print_status('Starting IRB shell...')
       print_status("You are in the \"self\" (session) object\n")
-      Rex::Ui::Text::Shell::HistoryManager.with_context(name: :irb) do
+      framework.history_manager.with_context(name: :irb) do
         Rex::Ui::Text::IrbShell.new(self).run
       end
     else
@@ -572,7 +571,7 @@ Shell Banner:
   # Open the Pry debugger on the current session
   #
   def cmd_pry(*args)
-    if args.include?('-h')
+    if args.include?('-h') || args.include?('--help')
       cmd_pry_help
       return
     end
@@ -587,7 +586,7 @@ Shell Banner:
     print_status('Starting Pry shell...')
     print_status("You are in the \"self\" (session) object\n")
     Pry.config.history_load = false
-    Rex::Ui::Text::Shell::HistoryManager.with_context(history_file: Msf::Config.pry_history, name: :pry) do
+    framework.history_manager.with_context(history_file: Msf::Config.pry_history, name: :pry) do
       self.pry
     end
   end
@@ -748,7 +747,7 @@ protected
   # shell_write instead of operating on rstream directly.
   def _interact
     framework.events.on_session_interact(self)
-    Rex::Ui::Text::Shell::HistoryManager.with_context(name: self.type.to_sym) {
+    framework.history_manager.with_context(name: self.type.to_sym) {
       _interact_stream
     }
   end
@@ -799,7 +798,7 @@ protected
   end
 
   def _file_transfer
-    raise NotImplementedError.new('Session does not support file transfers.') if @session_type.ends_with?(':winpty')
+    raise NotImplementedError.new('Session does not support file transfers.') if session_type.ends_with?(':winpty')
 
     FileTransfer.new(self)
   end
