@@ -134,7 +134,7 @@ module Msf
             res
           end
 
-          # Sends a kerberos AS request and reads the response
+          # Sends a kerberos TGS request and reads the response
           #
           # @param opts [Hash]
           # @return [Rex::Proto::Kerberos::Model::KdcResponse]
@@ -285,15 +285,49 @@ module Msf
               ),
             )
 
-            initial_as_res = send_request_as(req: initial_as_req)
+            req_opts = {req: initial_as_req}
+            req_opts.update(options)
+            initial_as_res = send_request_as(req_opts)
 
             # If we receive an AS_REP response immediately, no-preauthentication was required and we can return immediately
             if initial_as_res.msg_type == Rex::Proto::Kerberos::Model::AS_REP
+              pa_data = initial_as_res.pa_data
+              etype_entries = pa_data.find {|entry| entry.type == Rex::Proto::Kerberos::Model::PreAuthType::PA_ETYPE_INFO2}
+              if password.nil? && key.nil?
+                decrypted_part = nil
+                krb_enc_key = nil
+              else
+                # Let's try to check the password
+                server_ciphers = etype_entries.decoded_value
+                # Should only have one etype
+                etype_info = server_ciphers.etype_info2_entries[0]
+                if password
+                  enc_key, salt = get_enc_key_from_password(password, etype_info)
+                elsif key
+                  enc_key = key
+                end
+                begin
+                  decrypted_part = decrypt_kdc_as_rep_enc_part(
+                    initial_as_res,
+                    enc_key,
+                  )
+                  krb_enc_key = {
+                    enctype: etype_info.etype,
+                    key: enc_key,
+                    salt: salt
+                  }
+                rescue ::Rex::Proto::Kerberos::Model::Error::KerberosError
+                  # It's as if it were an invalid password
+                  decrypted_part = nil
+                  krb_enc_key = nil
+                end
+              end
+
               return Msf::Exploit::Remote::Kerberos::Model::TgtResponse.new(
                 as_rep: initial_as_res,
                 preauth_required: false,
-                decrypted_part: nil,
-                krb_enc_key: nil,
+                decrypted_part: decrypted_part,
+                krb_enc_key: krb_enc_key
               )
             end
 
@@ -334,12 +368,7 @@ module Msf
               selected_etype = selected_etypeinfo.etype
 
               if password
-                salt = selected_etypeinfo.salt
-                salt = salt.dup.force_encoding('utf-8') if salt
-                params = selected_etypeinfo.s2kparams
-
-                encryptor = Rex::Proto::Kerberos::Crypto::Encryption::from_etype(selected_etype)
-                enc_key = encryptor.string_to_key(password, salt, params: params)
+                enc_key, salt = get_enc_key_from_password(password, selected_etypeinfo)
               elsif key
                 raise ArgumentError.new('Encryption key provided without one offered encryption type') unless options[:offered_etypes]&.length == 1
                 enc_key = key
@@ -365,7 +394,9 @@ module Msf
                 )
               )
 
-              preauth_as_res = send_request_as(req: preauth_as_req)
+              req_opts = {req: preauth_as_req}
+              req_opts.update(options)
+              preauth_as_res = send_request_as(req_opts)
 
               # If we've succeeded - break out of trying ciphers
               break if preauth_as_res.msg_type == Rex::Proto::Kerberos::Model::AS_REP
@@ -401,6 +432,25 @@ module Msf
           def framework_module
             self
           end
+
+          private
+
+          #
+          # Construct the encryption key based on the etype_info passed
+          # @param password [String] The password to generate an encryption key for
+          # @param etype_info [Rex::Proto::Kerberos::Model::PreAuthEtypeInfo2Entry]
+          # @return [Array] The encryption key and the salt
+          #
+          def get_enc_key_from_password(password, etype_info)
+            salt = etype_info.salt
+            salt = salt.dup.force_encoding('utf-8') if salt
+            params = etype_info.s2kparams
+
+            encryptor = Rex::Proto::Kerberos::Crypto::Encryption::from_etype(etype_info.etype)
+            enc_key = encryptor.string_to_key(password, salt, params: params)
+
+            [enc_key, salt]
+          end
         end
       end
     end
