@@ -69,6 +69,10 @@ class Msf::Exploit::Remote::Kerberos::ServiceAuthenticator::Base
   #   @return [String] whether to send delegated creds (from the set Msf::Exploit::Remote::Kerberos::ServiceAuthenticator::Base::Delegation)
   attr_reader :send_delegated_creds
 
+  # @!attribute [r] dce_style
+  #   @return [Boolean] Whether this encryptor will be used for DCERPC purposes (since the behaviour is subtly different)
+  attr_reader :dce_style
+
   # @!attribute [r] ticket_storage
   #   @return [Msf::Exploit::Remote::Kerberos::Ticket::Storage::Base] the ticket storage driver
   attr_reader :ticket_storage
@@ -86,15 +90,17 @@ class Msf::Exploit::Remote::Kerberos::ServiceAuthenticator::Base
                 :print_status,
                 :print_good,
                 :vprint_error,
+                :vprint_status,
                 :workspace
 
   # Flags - https://datatracker.ietf.org/doc/html/rfc4121#section-4.1.1.1
-  GSS_DELEGATE      = 1
-  GSS_MUTUAL        = 2
-  GSS_REPLAY_DETECT = 4
-  GSS_SEQUENCE      = 8
-  GSS_CONFIDENTIAL  = 16
-  GSS_INTEGRITY     = 32
+  GSS_DELEGATE      = 0x01
+  GSS_MUTUAL        = 0x02
+  GSS_REPLAY_DETECT = 0x04
+  GSS_SEQUENCE      = 0x08
+  GSS_CONFIDENTIAL  = 0x10
+  GSS_INTEGRITY     = 0x20
+  GSS_DCE_STYLE     = 0x1000
 
   module Delegation
     ALWAYS = 'always' # Always send delegated creds
@@ -117,6 +123,7 @@ class Msf::Exploit::Remote::Kerberos::ServiceAuthenticator::Base
       use_gss_checksum: false,
       mechanism: Rex::Proto::Gss::Mechanism::SPNEGO,
       send_delegated_creds: Delegation::ALWAYS,
+      dce_style: false,
       cache_file: nil,
       ticket_storage: nil,
       key: nil,
@@ -138,6 +145,7 @@ class Msf::Exploit::Remote::Kerberos::ServiceAuthenticator::Base
     @use_gss_checksum = use_gss_checksum
     @mechanism = mechanism
     @send_delegated_creds = send_delegated_creds
+    @dce_style = dce_style
     @ticket_storage = ticket_storage || Msf::Exploit::Remote::Kerberos::Ticket::Storage::ReadWrite.new(
       framework: framework,
       framework_module: framework_module
@@ -183,6 +191,23 @@ class Msf::Exploit::Remote::Kerberos::ServiceAuthenticator::Base
     port
   end
 
+  def connect(options = {})
+    unless options[:rhost]
+      unless (host = @host)
+        vprint_status("Using DNS to lookup the KDC for #{realm}...")
+        host = ::Rex::Socket.getresources("_kerberos._tcp.#{realm}", :SRV)&.sample
+        if host.nil?
+          raise ::Rex::Proto::Kerberos::Model::Error::KerberosError.new("Failed to lookup the KDC")
+        end
+        print_status("Using KDC #{host} for realm #{realm}")
+        @host = host
+      end
+      options[:rhost] = host
+    end
+
+    super(options)
+  end
+
   # @param [Hash] options
   # @option options [String] :credential An explicit credential object to use for authentication.
   # @option options [Rex::Proto::Kerberos::Model::PrincipalName] :sname The target service principal name.
@@ -198,22 +223,27 @@ class Msf::Exploit::Remote::Kerberos::ServiceAuthenticator::Base
       else
         # load a cached TGS
         options[:credential] = get_cached_credential(options)
+        tgt_sname = Rex::Proto::Kerberos::Model::PrincipalName.new(
+          name_type: Rex::Proto::Kerberos::Model::NameType::NT_SRV_INST,
+          name_string: [
+            "krbtgt",
+            realm
+          ]
+        )
         unless options[:credential]
-          # load a cached TGT
+          # load a cached TGT (specific host)
           options[:credential] = get_cached_credential(
-            options.merge(
-              sname: Rex::Proto::Kerberos::Model::PrincipalName.new(
-                name_type: Rex::Proto::Kerberos::Model::NameType::NT_SRV_INST,
-                name_string: [
-                  "krbtgt",
-                  realm
-                ]
-              )
-            )
+            options.merge(sname: tgt_sname)
+          )
+        end
+        unless options[:credential]
+          # load a cached TGT (any host)
+          options[:credential] = get_cached_credential(
+            options.merge(sname: tgt_sname, host: nil)
           )
         end
         if options[:credential]
-          print_status("#{peer} - Using cached credential for #{options[:credential].server} #{options[:credential].client}")
+          print_status("Using cached credential for #{options[:credential].server} #{options[:credential].client}")
         end
       end
     end
@@ -247,7 +277,8 @@ class Msf::Exploit::Remote::Kerberos::ServiceAuthenticator::Base
                                             client_sequence_number,
                                             server_sequence_number,
                                             is_initiator: true,
-                                            use_acceptor_subkey: true)
+                                            use_acceptor_subkey: true,
+                                            dce_style: @dce_style)
   end
 
   def parse_gss_init_response(token, session_key, mechanism: 'kerberos')
@@ -258,16 +289,17 @@ class Msf::Exploit::Remote::Kerberos::ServiceAuthenticator::Base
       data = encapsulated_token[2, encapsulated_token.length]
       case tok_id
       when TOK_ID_KRB_AP_REP
-        ap_req = Rex::Proto::Kerberos::Model::ApRep.decode(data)
+        ap_rep = Rex::Proto::Kerberos::Model::ApRep.decode(data)
         print_good("#{peer} - Received AP-REQ. Extracting session key...")
 
-        raise ::Rex::Proto::Kerberos::Model::Error::KerberosError, 'Mismatching etypes' if session_key.type != ap_req.enc_part.etype
+        raise ::Rex::Proto::Kerberos::Model::Error::KerberosError, 'Mismatching etypes' if session_key.type != ap_rep.enc_part.etype
 
-        decrypted = ap_req.decrypt_enc_part(session_key.value)
+        decrypted = ap_rep.decrypt_enc_part(session_key.value)
 
         result = {
           ap_rep_subkey: decrypted.subkey,
-          server_sequence_number: decrypted.sequence_number
+          server_sequence_number: decrypted.sequence_number,
+          etype: ap_rep.enc_part.etype
         }
       when TOK_ID_KRB_ERROR
         krb_err = Rex::Proto::Kerberos::Model::KrbError.decode(data)
@@ -284,13 +316,19 @@ class Msf::Exploit::Remote::Kerberos::ServiceAuthenticator::Base
   end
 
   # @param security_blob [String] SPNEGO GSS Blob
-  # @raise [Rex::Proto::Kerberos::Model::Error::KerberosDecodingError] if the response was not successful
-  def validate_response!(security_blob)
-    gss_api = OpenSSL::ASN1.decode(security_blob)
-    neg_result = ::RubySMB::Gss.asn1dig(gss_api, 0, 0, 0)&.value.to_i
-    supported_neg = ::RubySMB::Gss.asn1dig(gss_api, 0, 1, 0)&.value
+  # @param accept_incomplete [Boolean] Whether an Incomplete value is an acceptable response
+  # @raise [Rex::Proto::Kerberos::Model::Error::KerberosError] if the response was not successful
+  # @raise [Rex::Proto::Kerberos::Model::Error::KerberosDecodingError] if the response was invalid per the Kerberos/GSS protocol
+  def validate_response!(security_blob, accept_incomplete: false)
+    begin
+      gss_api = OpenSSL::ASN1.decode(security_blob)
+      neg_result = ::RubySMB::Gss.asn1dig(gss_api, 0, 0, 0)&.value.to_i
+      supported_neg = ::RubySMB::Gss.asn1dig(gss_api, 0, 1, 0)&.value
+    rescue OpenSSL::ASN1::ASN1Error
+      raise ::Rex::Proto::Kerberos::Model::Error::KerberosDecodingError.new('Invalid GSS Response')
+    end
 
-    is_success = neg_result == NEG_TOKEN_ACCEPT_COMPLETED &&
+    is_success = (neg_result == NEG_TOKEN_ACCEPT_COMPLETED || (accept_incomplete && neg_result == NEG_TOKEN_ACCEPT_INCOMPLETE)) &&
       supported_neg == ::Rex::Proto::Gss::OID_MICROSOFT_KERBEROS_5.value
 
     raise ::Rex::Proto::Kerberos::Model::Error::KerberosError.new('Failed to negotiate Kerberos GSS') unless is_success
@@ -325,7 +363,7 @@ class Msf::Exploit::Remote::Kerberos::ServiceAuthenticator::Base
     end
 
     if credential
-      print_status("#{peer} - Using cached credential for #{credential.server} #{credential.client}")
+      print_status("Using cached credential for #{credential.server} #{credential.client}")
       return credential
     end
 
@@ -343,7 +381,7 @@ class Msf::Exploit::Remote::Kerberos::ServiceAuthenticator::Base
   def request_tgs_only(credential, options = {})
     # load a cached TGS
     if (ccache = get_cached_credential(options))
-      print_status("#{peer} - Using cached credential for #{ccache.server} #{ccache.client}")
+      print_status("Using cached credential for #{ccache.server} #{ccache.client}")
       return ccache
     end
 
@@ -467,6 +505,8 @@ class Msf::Exploit::Remote::Kerberos::ServiceAuthenticator::Base
 
   # Request a service ticket to a user on behalf of themselves
   # This is mostly useful for PKINIT to recover the NT hash
+  # Can combine this with S4U2Self by providing an :impersonate option
+  # to retrieve a PAC for any account, i.e. Sapphire Ticket attack
   #
   # @see https://learn.microsoft.com/en-us/archive/blogs/openspecification/how-kerberos-user-to-user-authentication-works
   #
@@ -504,6 +544,16 @@ class Msf::Exploit::Remote::Kerberos::ServiceAuthenticator::Base
       additional_tickets: [ticket]
     }
 
+    if options[:impersonate]
+      tgs_options[:pa_data] = build_pa_for_user(
+        {
+          username: options[:impersonate],
+          session_key: session_key,
+          realm: self.realm
+        }
+      )
+    end
+
     request_service_ticket(
       session_key,
       ticket,
@@ -563,7 +613,7 @@ class Msf::Exploit::Remote::Kerberos::ServiceAuthenticator::Base
       )
     end
 
-    if !tgt_result.preauth_required
+    if tgt_result.decrypted_part.nil? && !tgt_result.preauth_required
       raise ::Rex::Proto::Kerberos::Model::Error::KerberosError.new(
         'Kerberos ticket does not require preauthentication. It is not possible to decrypt the encrypted message to request further TGS tickets. Try cracking the password via AS-REP Roasting techniques.',
       )
@@ -616,7 +666,7 @@ class Msf::Exploit::Remote::Kerberos::ServiceAuthenticator::Base
 
     ## Service Authentication
     checksum = nil
-    checksum = build_gss_ap_req_checksum_value(mutual_auth, nil, nil, nil, nil, nil) if use_gss_checksum
+    checksum = build_gss_ap_req_checksum_value(mutual_auth, dce_style, nil, nil, nil, nil, nil) if use_gss_checksum
 
     sequence_number = rand(1 << 32)
     service_ap_request = build_service_ap_request(
@@ -706,6 +756,7 @@ class Msf::Exploit::Remote::Kerberos::ServiceAuthenticator::Base
     if use_gss_checksum
       checksum = build_gss_ap_req_checksum_value(
         mutual_auth,
+        dce_style,
         delegated_tgs_ticket,
         delegated_tgs_auth,
         tgs_auth.key,
@@ -740,7 +791,7 @@ class Msf::Exploit::Remote::Kerberos::ServiceAuthenticator::Base
     }
   end
 
-  def build_gss_ap_req_checksum_value(mutual_auth, ticket, decrypted_part, session_key, realm, client_name)
+  def build_gss_ap_req_checksum_value(mutual_auth, dce_style, ticket, decrypted_part, session_key, realm, client_name)
     # @see https://datatracker.ietf.org/doc/html/rfc4121#section-4.1.1
     # No channel binding
     channel_binding_info = "\x00" * 16
@@ -748,6 +799,7 @@ class Msf::Exploit::Remote::Kerberos::ServiceAuthenticator::Base
 
     flags = GSS_REPLAY_DETECT | GSS_SEQUENCE | GSS_CONFIDENTIAL | GSS_INTEGRITY
     flags |= GSS_MUTUAL if mutual_auth
+    flags |= GSS_DCE_STYLE if dce_style
     flags |= GSS_DELEGATE if ticket
 
     flags = [flags].pack('V')
@@ -977,7 +1029,7 @@ class Msf::Exploit::Remote::Kerberos::ServiceAuthenticator::Base
   def get_cached_credential(options = {})
     driver = options.fetch(:ticket_storage, @ticket_storage)
     driver.load_credential(
-      host: rhost,
+      host: options.fetch(:host) { rhost },
       client: options.fetch(:username) { self.username },
       server: options.fetch(:sname, nil),
       realm: options.fetch(:realm) { self.realm }
@@ -1030,7 +1082,9 @@ class Msf::Exploit::Remote::Kerberos::ServiceAuthenticator::Base
         next
       end
 
-      unless !sname_hostname || sname_hostname.to_s.casecmp?(credential.server.components[1])
+      unless !sname_hostname ||
+        sname_hostname.to_s.downcase == credential.server.components[1] ||
+        sname_hostname.to_s.downcase.ends_with?('.' + credential.server.components[1])
         wlog("Filtered credential #{file_path} ##{index} reason: SPN (#{sname_hostname}) hostname does not match (spn: #{credential.server.components.snapshot.join('/')})")
         next
       end
