@@ -5,11 +5,13 @@
 #
 
 require 'rex/proto/ldap'
+require 'metasploit/framework/ldap/client'
 
 module Msf
   module Exploit::Remote::LDAP
     include Msf::Exploit::Remote::Kerberos::Ticket::Storage
     include Msf::Exploit::Remote::Kerberos::ServiceAuthenticator::Options
+    include Metasploit::Framework::LDAP::Client
 
     # Initialize the LDAP client and set up the LDAP specific datastore
     # options to allow the client to perform authentication and timeout
@@ -72,148 +74,56 @@ module Msf
     # @return [Hash] The options to use when connecting to the target
     #    LDAP server.
     def get_connect_opts
-      connect_opts = {
-        host: rhost,
-        port: rport,
+      opts = {
+        username: datastore['USERNAME'],
+        password: datastore['PASSWORD'],
+        domain: datastore['DOMAIN'],
+        domain_controller_rhost: datastore['DomainControllerRhost'],
+        ldap_auth: datastore['LDAP::Auth'],
+        ldap_cert_file: datastore['LDAP::CertFile'],
+        ldap_rhostname: datastore['Ldap::Rhostname'],
+        ldap_krb_offered_enc_types: datastore['Ldap::KrbOfferedEncryptionTypes'],
+        ldap_krb5_cname: datastore['Ldap::Krb5Ccname'],
         proxies: datastore['Proxies'],
-        connect_timeout: datastore['LDAP::ConnectTimeout']
+        framework_module: self
       }
 
-      if datastore['SSL']
-        connect_opts[:encryption] = {
-          method: :simple_tls,
-          tls_options: {
-            verify_mode: OpenSSL::SSL::VERIFY_NONE
-          }
-        }
-      end
+      ldap_connect_opts(rhost, rport, datastore['LDAP::ConnectTimeout'], ssl: datastore['SSL'], opts: opts)
+    end
 
-      case datastore['LDAP::Auth']
-      when Msf::Exploit::Remote::AuthOption::SCHANNEL
-        pfx_path = datastore['LDAP::CertFile']
-        fail_with(Msf::Exploit::Remote::Failure::BadConfig, 'The LDAP::CertFile option is required when using SCHANNEL authentication.') if pfx_path.blank?
-        fail_with(Msf::Exploit::Remote::Failure::BadConfig, 'The SSL option must be enabled when using SCHANNEL authentication.') if datastore['SSL'] != true
-
-        unless ::File.file?(pfx_path) and ::File.readable?(pfx_path)
-          fail_with(Msf::Exploit::Remote::Failure::BadConfig, 'Failed to load the PFX certificate file. The path was not a readable file.')
-        end
-
-        begin
-          pkcs = OpenSSL::PKCS12.new(File.binread(pfx_path), '')
-        rescue => e
-          fail_with(Msf::Exploit::Remote::Failure::BadConfig, "Failed to load the PFX file (#{e})")
-        end
-
-        connect_opts[:auth] = {
-          method: :sasl,
-          mechanism: 'EXTERNAL',
-          initial_credential: '',
-          challenge_response: true
-        }
-        connect_opts[:encryption] = {
-          method: :start_tls,
-          tls_options: {
-            verify_mode: OpenSSL::SSL::VERIFY_NONE,
-            cert: pkcs.certificate,
-            key: pkcs.key
-          }
-        }
-      when Msf::Exploit::Remote::AuthOption::KERBEROS
-        fail_with(Msf::Exploit::Failure::BadConfig, 'The Ldap::Rhostname option is required when using Kerberos authentication.') if datastore['Ldap::Rhostname'].blank?
-        fail_with(Msf::Exploit::Failure::BadConfig, 'The DOMAIN option is required when using Kerberos authentication.') if datastore['DOMAIN'].blank?
-        fail_with(Msf::Exploit::Failure::BadConfig, 'The DomainControllerRhost is required when using Kerberos authentication.') if datastore['DomainControllerRhost'].blank?
-        offered_etypes = Msf::Exploit::Remote::AuthOption.as_default_offered_etypes(datastore['Ldap::KrbOfferedEncryptionTypes'])
-        fail_with(Msf::Exploit::Failure::BadConfig, 'At least one encryption type is required when using Kerberos authentication.') if offered_etypes.empty?
-
-        kerberos_authenticator = Msf::Exploit::Remote::Kerberos::ServiceAuthenticator::LDAP.new(
-          host: datastore['DomainControllerRhost'],
-          hostname: datastore['Ldap::Rhostname'],
-          proxies: datastore['Proxies'],
-          realm: datastore['DOMAIN'],
-          username: datastore['USERNAME'],
-          password: datastore['PASSWORD'],
-          framework: framework,
-          framework_module: self,
-          cache_file: datastore['Ldap::Krb5Ccname'].blank? ? nil : datastore['Ldap::Krb5Ccname'],
-          ticket_storage: kerberos_ticket_storage,
-          offered_etypes: offered_etypes
-        )
-
-        kerberos_result = kerberos_authenticator.authenticate
-
-        connect_opts[:auth] = {
-          method: :sasl,
-          mechanism: 'GSS-SPNEGO',
-          initial_credential: kerberos_result[:security_blob],
-          challenge_response: true
-        }
-      when Msf::Exploit::Remote::AuthOption::NTLM
-        ntlm_client = RubySMB::NTLM::Client.new(
-          datastore['USERNAME'],
-          datastore['PASSWORD'],
-          workstation: 'WORKSTATION',
-          domain: datastore['DOMAIN'].blank? ? '.' : datastore['DOMAIN'],
-          flags:
-            RubySMB::NTLM::NEGOTIATE_FLAGS[:UNICODE] |
-            RubySMB::NTLM::NEGOTIATE_FLAGS[:REQUEST_TARGET] |
-            RubySMB::NTLM::NEGOTIATE_FLAGS[:NTLM] |
-            RubySMB::NTLM::NEGOTIATE_FLAGS[:ALWAYS_SIGN] |
-            RubySMB::NTLM::NEGOTIATE_FLAGS[:EXTENDED_SECURITY] |
-            RubySMB::NTLM::NEGOTIATE_FLAGS[:KEY_EXCHANGE] |
-            RubySMB::NTLM::NEGOTIATE_FLAGS[:TARGET_INFO] |
-            RubySMB::NTLM::NEGOTIATE_FLAGS[:VERSION_INFO]
-        )
-
-        negotiate = proc do |challenge|
-          ntlmssp_offset = challenge.index('NTLMSSP')
-          type2_blob = challenge.slice(ntlmssp_offset..-1)
-          challenge = [type2_blob].pack('m')
-          type3_message = ntlm_client.init_context(challenge)
-          type3_message.serialize
-        end
-
-        connect_opts[:auth] = {
-          method: :sasl,
-          mechanism: 'GSS-SPNEGO',
-          initial_credential: ntlm_client.init_context.serialize,
-          challenge_response: negotiate
-        }
-      when Msf::Exploit::Remote::AuthOption::PLAINTEXT
-        username = datastore['USERNAME'].dup
-        username << "@#{datastore['DOMAIN']}" unless datastore['DOMAIN'].blank?
-        connect_opts[:auth] = {
-          method: :simple,
-          username: username,
-          password: datastore['PASSWORD']
-        }
-      when Msf::Exploit::Remote::AuthOption::AUTO
-        unless datastore['USERNAME'].blank? # plaintext if specified
-          username = datastore['USERNAME'].dup
-          username << "@#{datastore['DOMAIN']}" unless datastore['DOMAIN'].blank?
-          connect_opts[:auth] = {
-            method: :simple,
-            username: username,
-            password: datastore['PASSWORD']
-          }
-        end
-      end
-
-      connect_opts
+    # @see #ldap_open
+    # @return [Object] The result of whatever the block that was
+    #   passed in via the "block" parameter yielded.
+    def ldap_connect(opts = {}, &block)
+      ldap_open(get_connect_opts.merge(opts), &block)
     end
 
     # Connect to the target LDAP server using the options provided,
     # and pass the resulting connection object to the proc provided.
     # Terminate the connection once the proc finishes executing.
     #
-    # @param opts [Hash] Options for the LDAP connection.
+    # @param connect_opts [Hash] Options for the LDAP connection.
     # @param block [Proc] A proc containing the functionality to execute
     #   after the LDAP connection has succeeded. The connection is closed
     #   once this proc finishes executing.
     # @see Net::LDAP.open
     # @return [Object] The result of whatever the block that was
     #   passed in via the "block" parameter yielded.
-    def ldap_connect(opts = {}, &block)
-      Net::LDAP.open(get_connect_opts.merge(opts), &block)
+    def ldap_open(connect_opts, &block)
+      opts = resolve_connect_opts(connect_opts)
+      Net::LDAP.open(opts, &block)
+    end
+
+
+    def resolve_connect_opts(connect_opts)
+      return connect_opts unless connect_opts.dig(:auth, :initial_credential).is_a?(Proc)
+
+      opts = connect_opts.dup
+      # For scenarios such as Kerberos, we might need to make additional calls out to a separate services to acquire an initial credential
+      opts[:auth].merge!(
+        initial_credential: opts[:auth][:initial_credential].call
+      )
+      opts
     end
 
     # Create a new LDAP connection using Net::LDAP.new and yield the
@@ -224,7 +134,8 @@ module Msf
     # @yieldparam ldap [Net::LDAP] The LDAP connection handle to use for connecting to
     #   the target LDAP server.
     def ldap_new(opts = {})
-      ldap = Net::LDAP.new(get_connect_opts.merge(opts))
+
+      ldap = Net::LDAP.new(resolve_connect_opts(get_connect_opts.merge(opts)))
 
       # NASTY, but required
       # monkey patch ldap object in order to ignore bind errors
@@ -297,7 +208,7 @@ module Msf
       end
 
       # NOTE: Find the first entry that starts with `DC=` as this will likely be the base DN.
-      naming_contexts.select! { |context| context =~ /^(DC=[A-Za-z0-9-]+,?)+$/ }
+      naming_contexts.select! { |context| context =~ /^([Dd][Cc]=[A-Za-z0-9-]+,?)+$/ }
       naming_contexts.reject! { |context| context =~ /(Configuration)|(Schema)|(ForestDnsZones)/ }
       if naming_contexts.blank?
         print_error("#{peer} A base DN matching the expected format could not be found!")
