@@ -94,9 +94,9 @@ module Msf
       #
       # @param (see Exploit::Remote::Tcp#connect)
       # @return (see Exploit::Remote::Tcp#connect)
-      def connect(global=true, versions: [], backend: nil)
+      def connect(global=true, versions: [], backend: nil, direct: nil)
         if versions.nil? || versions.empty?
-          versions = datastore['SMB::ProtocolVersion'].split(',').map(&:to_i)
+          versions = datastore['SMB::ProtocolVersion'].split(',').map(&:strip).reject(&:blank?).map(&:to_i)
           # if the user explicitly set the protocol version to 1, still use ruby_smb
           backend ||= :ruby_smb if versions == [1]
         end
@@ -108,9 +108,11 @@ module Msf
 
         # Disable direct SMB when SMBDirect has not been set
         # and the destination port is configured as 139
-        direct = smb_direct
-        if(datastore.default?('SMBDirect') and rport.to_i == 139)
-          direct = false
+        if direct.nil?
+          direct = smb_direct
+          if datastore.default?('SMBDirect') and rport.to_i == 139
+            direct = false
+          end
         end
 
         c = Rex::Proto::SMB::SimpleClient.new(s, direct, versions, always_encrypt: datastore['SMB::AlwaysEncrypt'], backend: backend)
@@ -153,12 +155,11 @@ module Msf
         if datastore['SMB::Auth'] == Msf::Exploit::Remote::AuthOption::KERBEROS
           fail_with(Msf::Exploit::Failure::BadConfig, 'The Smb::Rhostname option is required when using Kerberos authentication.') if datastore['Smb::Rhostname'].blank?
           fail_with(Msf::Exploit::Failure::BadConfig, 'The SMBDomain option is required when using Kerberos authentication.') if datastore['SMBDomain'].blank?
-          fail_with(Msf::Exploit::Failure::BadConfig, 'The DomainControllerRhost is required when using Kerberos authentication.') if datastore['DomainControllerRhost'].blank?
           offered_etypes = Msf::Exploit::Remote::AuthOption.as_default_offered_etypes(datastore['Smb::KrbOfferedEncryptionTypes'])
           fail_with(Msf::Exploit::Failure::BadConfig, 'At least one encryption type is required when using Kerberos authentication.') if offered_etypes.empty?
 
           kerberos_authenticator = Msf::Exploit::Remote::Kerberos::ServiceAuthenticator::SMB.new(
-            host: datastore['DomainControllerRhost'],
+            host: datastore['DomainControllerRhost'].blank? ? nil : datastore['DomainControllerRhost'],
             hostname: datastore['Smb::Rhostname'],
             proxies: datastore['Proxies'],
             realm: datastore['SMBDomain'],
@@ -756,7 +757,7 @@ module Msf
         share_info
       end
 
-      # Retreive a list of all shares using any available method
+      # Retrieve a list of all shares using any available method
       def smb_netshareenumall
         begin
           return smb_srvsvc_netshareenumall
