@@ -52,9 +52,9 @@ class MetasploitModule < Msf::Auxiliary
           'SideEffects' => [ CONFIG_CHANGES ], # This module will change the configuration by
           # resetting the router to the default factory password.
           'Stability' => [ CRASH_SERVICE_DOWN ], # This module will crash the target service after it is run.
-          'Reliability' => []
+          'Reliability' => [],
+          'RelatedModules' => [ 'exploit/linux/telnet/netgear_telnetenable' ], # This module relies on users also running exploit/linux/telnet/netgear_telnetenable to get the shell.
         },
-        'RelatedModules' => [ 'exploit/linux/telnet/netgear_telnetenable' ], # This module relies on users also running exploit/linux/telnet/netgear_telnetenable to get the shell.
         'DisclosureDate' => '2020-06-15',
         'DefaultTarget' => 0
       )
