@@ -14,6 +14,8 @@ class MetasploitModule < Msf::Auxiliary
   include Msf::Auxiliary::Scanner
   include Msf::Auxiliary::Report
   include Msf::Auxiliary::AuthBrute
+  include Msf::Auxiliary::CommandShell
+  include Msf::Sessions::CreateSessionOptions
 
   Aliases = [
     'auxiliary/scanner/smb/login'
@@ -45,7 +47,8 @@ class MetasploitModule < Msf::Auxiliary
       'DefaultOptions' => {
         'DB_ALL_CREDS' => false,
         'BLANK_PASSWORDS' => false,
-        'USER_AS_PASS' => false
+        'USER_AS_PASS' => false,
+        'CreateSession' => false
       }
     )
 
@@ -58,11 +61,45 @@ class MetasploitModule < Msf::Auxiliary
         OptBool.new('PRESERVE_DOMAINS', [ false, 'Respect a username that contains a domain name.', true ]),
         OptBool.new('RECORD_GUEST', [ false, 'Record guest-privileged random logins to the database', false ]),
         OptBool.new('DETECT_ANY_AUTH', [false, 'Enable detection of systems accepting any authentication', false]),
-        OptBool.new('DETECT_ANY_DOMAIN', [false, 'Detect if domain is required for the specified user', false])
+        OptBool.new('DETECT_ANY_DOMAIN', [false, 'Detect if domain is required for the specified user', false]),
+        OptBool.new('CreateSession', [false, 'Create a new session for every successful login', false])
       ]
     )
 
-    deregister_options('USERNAME', 'PASSWORD', 'PASSWORD_SPRAY')
+    options_to_deregister = %w[USERNAME PASSWORD PASSWORD_SPRAY CommandShellCleanupCommand AutoVerifySession]
+
+    if framework.features.enabled?(Msf::FeatureManager::SMB_SESSION_TYPE)
+      add_info('New in Metasploit 6.4 - The %grnCreateSession%clr option within this module can open an interactive session')
+    else
+      # Don't give the option to create a session unless smb sessions are enabled
+      options_to_deregister << 'CreateSession'
+    end
+
+    deregister_options(*options_to_deregister)
+
+  end
+
+  def create_session?
+    # The CreateSession option is de-registered if SMB_SESSION_TYPE is not enabled
+    # but the option can still be set/saved so check to see if we should use it
+    if framework.features.enabled?(Msf::FeatureManager::SMB_SESSION_TYPE)
+      datastore['CreateSession']
+    else
+      false
+    end
+  end
+
+  def run
+    results = super
+    logins = results.flat_map { |_k, v| v[:successful_logins] }
+    sessions = results.flat_map { |_k, v| v[:successful_sessions] }
+    print_status("Bruteforce completed, #{logins.size} #{logins.size == 1 ? 'credential was' : 'credentials were'} successful.")
+    if datastore['CreateSession']
+      print_status("#{sessions.size} SMB #{sessions.size == 1 ? 'session was' : 'sessions were'} opened successfully.")
+    else
+      print_status('You can open an SMB session with these credentials and %grnCreateSession%clr set to true')
+    end
+    results
   end
 
   def run_host(ip)
@@ -87,7 +124,8 @@ class MetasploitModule < Msf::Auxiliary
           framework: framework,
           framework_module: self,
           cache_file: datastore['Smb::Krb5Ccname'].blank? ? nil : datastore['Smb::Krb5Ccname'],
-          ticket_storage: kerberos_ticket_storage
+          # Write only cache so we keep all gathered tickets but don't reuse them for auth while running the module
+          ticket_storage: kerberos_ticket_storage({ read: false, write: true })
         )
       end
     end
@@ -104,7 +142,10 @@ class MetasploitModule < Msf::Auxiliary
       send_delay: datastore['TCP::send_delay'],
       framework: framework,
       framework_module: self,
-      kerberos_authenticator_factory: kerberos_authenticator_factory
+      always_encrypt: datastore['SMB::AlwaysEncrypt'],
+      versions: datastore['SMB::ProtocolVersion'].split(',').map(&:strip).reject(&:blank?).map(&:to_i),
+      kerberos_authenticator_factory: kerberos_authenticator_factory,
+      use_client_as_proof: create_session?
     )
 
     if datastore['DETECT_ANY_AUTH']
@@ -129,7 +170,8 @@ class MetasploitModule < Msf::Auxiliary
     cred_collection = prepend_db_hashes(cred_collection)
 
     @scanner.cred_details = cred_collection
-
+    successful_logins = []
+    successful_sessions = []
     @scanner.scan! do |result|
       case result.status
       when Metasploit::Model::Login::Status::LOCKED_OUT
@@ -146,7 +188,17 @@ class MetasploitModule < Msf::Auxiliary
         :next_user
       when Metasploit::Model::Login::Status::SUCCESSFUL
         print_brute level: :good, ip: ip, msg: "Success: '#{result.credential}' #{result.access_level}"
+        successful_logins << result
         report_creds(ip, rport, result)
+        if create_session?
+          begin
+            successful_sessions << session_setup(result)
+          rescue ::StandardError => e
+            elog('Failed to setup the session', error: e)
+            print_brute level: :error, ip: ip, msg: "Failed to setup the session - #{e.class} #{e.message}"
+            result.connection.close unless result.connection.nil?
+          end
+        end
         :next_user
       when Metasploit::Model::Login::Status::UNABLE_TO_CONNECT
         if datastore['VERBOSE']
@@ -181,6 +233,7 @@ class MetasploitModule < Msf::Auxiliary
         )
       end
     end
+    { successful_logins: successful_logins, successful_sessions: successful_sessions }
   end
 
   # This logic is not universal ie a local account will not care about workgroup
@@ -242,4 +295,22 @@ class MetasploitModule < Msf::Auxiliary
 
     create_credential_login(login_data)
   end
+
+  # @param [Metasploit::Framework::LoginScanner::Result] result
+  # @return [Msf::Sessions::SMB]
+  def session_setup(result)
+    return unless (result.connection && result.proof)
+
+    my_session = Msf::Sessions::SMB.new(result.connection, { client: result.proof })
+    merge_me = {
+      'USERPASS_FILE' => nil,
+      'USER_FILE'     => nil,
+      'PASS_FILE'     => nil,
+      'USERNAME'      => result.credential.public,
+      'PASSWORD'      => result.credential.private
+    }
+
+    start_session(self, nil, merge_me, false, my_session.rstream, my_session)
+  end
+
 end
