@@ -7,7 +7,8 @@ done
 ./msfconsole -qr tools/automation/cache/wait_for_cache.rc
 cp ~/.msf4/store/modules_metadata.json db/modules_metadata_base.json
 cp ~/.msf4/logs/framework.log .
-CACHE_CHANGE=`git diff db/modules_metadata_base.json`
-if [ -n "$CACHE_CHANGE" ]; then
+set +e
+git diff --exit-code db/modules_metadata_base.json
+if [ ! $? ]; then
   echo "Module cache updates exist."
 fi
